@@ -71,17 +71,26 @@
 #'
 #' @param ... Not currently used.
 #'
-#' @return A \code{\link{tibble}} with same number of rows as \code{newdata}
-#'   and in the same order. If multiple predictions are requested, a
-#'   \code{\link{tibble}} containing a single list-column of data frames.
+#' @return A \code{\link{tibble}} with same number of rows as
+#'   \code{newdata} and in the same order. If multiple predictions are
+#'   requested, a \code{\link{tibble}} containing a single list-column
+#'   of data frames.
 #'
 #'   For the list-column of data frames - the dimensions of each data frame
 #'   will be identical. Rows are added for each value of \code{times} or
 #'   \code{p} requested.
 #'
+#' This function is a wrapper around \code{\link{summary.flexsurvreg}},
+#' designed to help \pkg{flexsurv} to integrate with the "tidymodels"
+#' ecosystem, in particular the \pkg{censored} package.
+#' \code{\link{summary.flexsurvreg}} returns the same results but in a more
+#' conventional format. 
+#'
 #' @seealso \code{\link{summary.flexsurvreg}},
 #'   \code{\link{residuals.flexsurvreg}}
 #'
+#' @author Matthew T. Warkentin (\url{https://github.com/mattwarkentin})
+#' 
 #' @importFrom tibble tibble
 #' @importFrom stats predict
 #'
@@ -245,7 +254,7 @@ utils::globalVariables('.id')
 tidy_names <- function() {
     tibble::tibble(
         old = c("time", "quantile", "est", "se", "lcl", "ucl"),
-        new = c(".time", ".quantile", ".pred", ".std_error",
+        new = c(".eval_time", ".quantile", ".pred", ".std_error",
                 ".pred_lower", ".pred_upper")
     )
 }
