@@ -7,6 +7,18 @@ test_that("check tidy", {
   )
 })
 
+test_that("check tidy against survreg", {
+  fit_fs <- flexsurvreg(Surv(futime, fustat) ~ ecog.ps,
+                      data = ovarian, dist = "weibull")
+  fit_sr <- survreg(Surv(futime, fustat) ~ ecog.ps,
+                    data = ovarian, dist = "weibull")
+  expect_equal(
+    object = as.data.frame(tidy(fit_fs)[3, ]),
+    expected = as.data.frame(broom::tidy(fit_sr)[2, ]),
+    tolerance = 1e-6
+  )
+})
+
 test_that("check glance", {
   fitw <- flexsurvreg(Surv(futime, fustat) ~ age,
                       data = ovarian, dist = "weibull")
